<?php
namespace Env\Routing { final class Router { private $config, $devkit, $rules, $uri, $procedure, $storage, $request, $plugin_id; public $current_lang, $render; public function &procedure() { return $this->procedure; } public function &request() { return $this->request; } public function &storage() { return $this->storage; } public function &uri($pattern = false, $tokens = false) { if($pattern !== false) { if(is_array($tokens)) $url = new \Env\Uri\URL($pattern, $tokens); else $url = $this->uri->format($pattern); return $url; } else { return $this->uri; } } public function __construct(&$config, &$devkit, $filename, $plugin_id = false) { $this->config = &$config; $this->devkit = &$devkit; $this->plugin_id = $plugin_id; $this->request = new Request(); if(is_readable($filename)) { $xml = secure_simplexml_load_file($filename, $errors); if($xml) { \Artise::event()->trigger('env:router:loadXML', array(&$xml)); $this->rules = $this->parseRouting($xml); } else { die('E0039'); throw new \HunterException('E0039'); } } else { die('E0040'); throw new \HunterException('E0040'); } } private function parseRouting($rules_xml) { if(count($rules_xml) == 0) return false; $rules = array(); foreach($rules_xml as $rule) { try { $rules[] = new Rule($rule); } catch (\HunterException $e) { $this->devkit->hunter->addError($e); } } return $rules; } private function filterRequest() { $request = &$this->request; return array_filter($this->rules, function($rule) use (&$request) { if(isset($rule->request)) { if($rule->request == 'ajax' && $request->get() == AJAX_Request) return true; if($rule->request == 'std' && $request->get() == Standard_Request) return true; return false; } return $request->get() === Routing_Request_Default; }); } private function filterUri($uri, &$tokens) { return array_filter($this->rules, function(&$rule) use ($uri, &$tokens) { if(isset($rule->url)) { foreach($rule->url as $pattern) { if(\Env\Uri\Validator::validate($uri, new \Env\Uri\PatternParser($pattern), $tokens)) { $rule->matchURL($uri); return true; } } return false; } return false; }); } public function getPath($path) { if (!$this->plugin_id) return $path; $base_path = Artise_Path_Plugins . DS . $this->plugin_id; switch($path) { case Artise_Path_Renders: return $base_path . '/code/renders'; case Artise_Path_Actions: return $base_path . '/code/actions'; case Artise_Path_Painters: return $base_path . '/code/painters'; case Artise_Path_Layouts: return $base_path . '/design/layouts'; case Artise_Path_Palettes: return $base_path . '/design/palettes'; case Artise_Path_Langs: return $base_path . '/langs'; } } public function route($uri, &$lang) { if(!is_object($lang)) { if (!$this->plugin_id) $lang = new \Env\xLang\xLang(); $uri = \Env\Uri\Parser::Parse($this->config, $lang, $uri); } $this->rules = $this->filterRequest(); $tokens = array(); $rules = $this->filterUri($uri, $tokens); if(count($rules) === 0) { die('No rules matched'); } else { $selected_rule = false; foreach($rules as $rule) { if ($selected_rule === false) { $selected_rule = $rule; } else { if (substr_count($rule->matched_url, '/') > substr_count($selected_rule->matched_url, '/')) $selected_rule = $rule; } } if ($selected_rule->plugin && !$this->plugin_id) { $plugin_router = new Router($this->config, $this->devkit, Artise_Path_Plugins . DS . $selected_rule->plugin . DS . 'code' . DS . 'routing.xml', $selected_rule->plugin); $plugin_url = substr($uri, strlen($selected_rule->matched_url) - 1); $lang = new \Env\xLang\xLang($selected_rule->plugin); $uri = \Env\Uri\Parser::Parse($this->config, $lang, $uri); $plugin_router->route($plugin_url, $lang); return $plugin_router; } if ((string)$selected_rule->matched_url != $uri) { header("Location: " . $selected_rule->matched_url); } $this->render = $selected_rule->render; $this->request = new \Env\Routing\Request(); $this->uri = $selected_rule->matched_url; $uri_procedure = $this->uri->procedure ? $this->uri->procedure : ''; if(isset($selected_rule->procedures[$uri_procedure])) $this->procedure = $selected_rule->procedures[$uri_procedure]; $this->storage = new \Env\Routing\Storage($this->uri); } return true; } public function render(&$site, &$devkit) { try { if(file_exists($this->getPath(Artise_Path_Renders) . '/commons.func.php')) include $this->getPath(Artise_Path_Renders) . '/commons.func.php'; $path = $this->getPath(Artise_Path_Renders) . DS; $filename = $path . DS . $this->render . '.php'; if(!file_exists($filename)) throw new \HunterException( new \Env\Source\DummyIdiom('E0050', array('element' => 'Render')), new \Env\Source\DummyIdiom('file_path_abstract', array( 'path' => $path, 'filename' => $this->render)), 'TIP_E0050', 'router'); if(!is_readable($filename)) throw new \HunterException( new \Env\Source\DummyIdiom('E0051', array('element' => 'Render')), new \Env\Source\DummyIdiom('file_path_abstract', array( 'path' => $path, 'filename' => $this->render)), 'TIP_E0051'); include $filename; $parts = explode('/', $this->render); $class_name = ''; if (count($parts) > 1) { $class_name = $parts[count($parts) - 1]; unset($parts[count($parts) - 1]); $render_ns = 'renders\\' . implode('\\', $parts); } else { $class_name = $this->render; $render_ns = 'renders'; } $ns_path = '\\' . $render_ns . '\\'; $render_path = $ns_path . $class_name; if(class_exists($render_path, false)) { $render = new $render_path($site, $devkit); $render_parent_classes = array('Ext\Rendering\Page', 'Ext\Rendering\Layout', 'Ext\Rendering\Painter'); if(!in_array(get_parent_class($render), $render_parent_classes)) throw new \HunterException( new \Env\Source\DummyIdiom('E0060', array( 'element' => 'Render', 'name' => $class_name)), new \Env\Source\DummyIdiom('namespace_path_abstract', array('path' => $ns_path, 'filename' => $class_name)), new \Env\Source\DummyIdiom('TIP_E0060_A'), 'router'); } else { throw new \HunterException( new \Env\Source\DummyIdiom('E0053', array( 'element' => 'Render', 'name' => $class_name)), new \Env\Source\DummyIdiom('namespace_path_abstract', array('path' => $ns_path, 'filename' => $class_name)), new \Env\Source\DummyIdiom('TIP_W0053', array('component' => 'render')), 'router'); } return (string)$render; } catch (\HunterException $e) { \Artise::hunter()->warning($e); } } } } ?>
